using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace Smdn.Formats.Mime {
  public class MimeMessage {
    public Dictionary<string, string[]> Headers {
      get { return headers; }
    }

    public string Body {
      get { return body; }
    }

    public byte[] BodyBytes {
      get { return bodyBytes; }
      internal set { bodyBytes = value; }
    }

    public List<MimeMessage> SubParts {
      get { return subParts; }
    }

    public MimeMessage()
      : this(new Dictionary<string, string[]>(), string.Empty, new MimeMessage[] {})
    {
    }

    public MimeMessage(Dictionary<string, string[]> headers, string body, IEnumerable<MimeMessage> subParts)
    {
      foreach (var pair in headers) {
        this.headers.Add(pair.Key, pair.Value);
      }

      this.body = body;
      this.subParts.AddRange(subParts);
    }

    public override string ToString ()
    {
      return string.Format("{{Headers={0}, Body={1}, SubParts={2}}}",
                           headers.Count,
                           body,
                           subParts.Count);
    }

    public static MimeMessage Create(string str)
    {
      return Create(str, true);
    }

    public static MimeMessage Create(Stream stream)
    {
      return Create(stream, true);
    }

    public static MimeMessage Create(string str, bool decode)
    {
      return Create(new StringReader(str), decode);
    }

    public static MimeMessage Create(Stream stream, bool decode)
    {
      return Create(new StreamReader(stream, latin1Encoding), decode);
    }

    public static MimeMessage Create(TextReader reader)
    {
      return Create(reader, true);
    }

    public static MimeMessage Create(TextReader reader, bool decode)
    {
      var message = Parser.Parse(reader);

      if (decode)
        return Decoder.Decode(message);
      else
        return message;
    }

    public void Format(Stream stream)
    {
      Formatter.Format(Encoder.Encode(this), new StreamWriter(stream, Encoding.ASCII));
    }

    private readonly Dictionary<string, string[]> headers = new Dictionary<string,string[]>(StringComparer.InvariantCultureIgnoreCase);
    private string body = string.Empty;
    private byte[] bodyBytes = null;
    private readonly List<MimeMessage> subParts = new List<MimeMessage>();
    private static readonly Encoding latin1Encoding = Encoding.GetEncoding("ISO-8859-1");
  }
}