using System;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Protocols.Imap4 {
  public class ImapMailbox {
    public ImapServerInfo ServerInfo {
      get { return serverInfo; }
    }

    public ImapUri Uri {
      get { return ImapMailboxManager.GetMailboxUrl(this); }
    }

    public string Name {
      get { return name; }
    }

    public string HierarchyDelimiter {
      get { return hierarchyDelimiter; }
      internal set { hierarchyDelimiter = value; }
    }

    public bool ReadOnly {
      get { return readOnly; }
      internal set { readOnly = value; }
    }

    public ImapMailboxFlags MailboxFlags {
      get { return mailboxFlags; }
      internal set { mailboxFlags = value; }
    }

    public string[] ExtendedMailboxFlags {
      get { return extendedMailboxFlags; }
      internal set { extendedMailboxFlags = value; }
    }

    public ImapMessageFlags ApplicableFlags {
      get { return applicableFlags; }
      internal set { applicableFlags = value; }
    }

    public string[] ExtendedApplicableFlags {
      get { return extendedApplicableFlags; }
      internal set { extendedApplicableFlags = value; }
    }

    public ImapMessageFlags PermanentFlags {
      get { return permanentFlags; }
      internal set { permanentFlags = value; }
    }

    public string[] ExtendedPermanentFlags {
      get { return extendedPermanentFlags; }
      internal set { extendedPermanentFlags = value; }
    }

    public int ExistsMessage {
      get { return existsMessage; }
      internal set { existsMessage = value; }
    }

    public int RecentMessage {
      get { return recentMessage; }
      internal set { recentMessage = value; }
    }

    public int UnseenMessage {
      get { return unseenMessage; }
      internal set { unseenMessage = value; }
    }

    public int FirstUnseen {
      get { return firstUnseen; }
      internal set { firstUnseen = value; }
    }

    public int UidNext {
      get { return uidNext; }
      internal set { uidNext = value; }
    }

    public int UidValidity {
      get { return uidValidity; }
      internal set { uidValidity = value; }
    }

    internal ImapMailbox(ImapServerInfo serverInfo, string name)
      : this(serverInfo, name, true)
    {
    }

    internal ImapMailbox(ImapServerInfo serverInfo, string name, bool readOnly)
    {
      this.serverInfo = serverInfo;
      this.name = name;
      this.readOnly = readOnly;
    }

    public bool HasMailboxFlag(ImapMailboxFlags flag)
    {
      return (0 != (mailboxFlags & flag));
    }

    public bool HasMailboxFlag(string extendedFlag)
    {
      foreach (var mailboxFlag in extendedMailboxFlags) {
        if (extendedFlag.Equals(mailboxFlag, StringComparison.InvariantCultureIgnoreCase))
          return true;
      }

      return false;
    }

    public bool HasPermanentFlag(ImapMessageFlags flag)
    {
      return (0 != (permanentFlags & flag));
    }

    public bool HasPermanentFlag(string extendedFlag)
    {
      foreach (var permanentFlag in extendedPermanentFlags) {
        if (extendedFlag.Equals(permanentFlag, StringComparison.InvariantCultureIgnoreCase))
          return true;
      }

      return false;
    }

    public bool HasApplicableFlag(ImapMessageFlags flag)
    {
      return (0 != (applicableFlags & flag));
    }

    public bool HasApplicableFlag(string extendedFlag)
    {
      foreach (var applicableFlag in extendedApplicableFlags) {
        if (extendedFlag.Equals(applicableFlag, StringComparison.InvariantCultureIgnoreCase))
          return true;
      }

      return false;
    }

    public override string ToString()
    {
      return string.Format("{{Name={0}, HierarchyDelimiter={1}, ReadOnly={2}, MailboxFlags={3}, ApplicableFlags={4}, PermanentFlags={5}, ExistsMessage={6}, RecentMessage={7}, UnseenMessage={8}, FirstUnseen={9}, UidNext={10}, UidValidity={11}}}",
                           name,
                           hierarchyDelimiter,
                           readOnly,
                           mailboxFlags,
                           applicableFlags,
                           permanentFlags,
                           existsMessage,
                           recentMessage,
                           unseenMessage,
                           firstUnseen,
                           uidNext,
                           uidValidity);
    }

    private readonly ImapServerInfo serverInfo;
    private readonly string name;
    private string hierarchyDelimiter = string.Empty;
    private bool readOnly;
    private ImapMailboxFlags mailboxFlags = ImapMailboxFlags.None;
    private string[] extendedMailboxFlags = new string[] {};
    private ImapMessageFlags applicableFlags = ImapMessageFlags.None;
    private string[] extendedApplicableFlags = new string[] {};
    private ImapMessageFlags permanentFlags = ImapMessageFlags.None;
    private string[] extendedPermanentFlags = new string[] {};
    private int existsMessage = 0;
    private int recentMessage = 0;
    private int unseenMessage = 0;
    private int firstUnseen = 0;
    private int uidNext = 0;
    private int uidValidity = 0;
  }
}