using System;
using System.Collections.Generic;

namespace Smdn.Protocols.Imap4 {
  public class ImapMailboxHierarchy {
    private class HierarchyTree {
      public Dictionary<string, HierarchyTree> SubTrees = new Dictionary<string, HierarchyTree>();
      public ImapMailbox Mailbox;
    }

    public ImapMailboxHierarchy[] SubHierarchies {
      get { return subHierarchies; }
    }

    public ImapMailbox Mailbox {
      get { return mailbox; }
    }

    public string Name {
      get
      {
        if (string.IsNullOrEmpty(mailbox.HierarchyDelimiter))
          return mailbox.Name;
        else if (mailbox.Name.Contains(mailbox.HierarchyDelimiter))
          return mailbox.Name.Substring(mailbox.Name.LastIndexOf(mailbox.HierarchyDelimiter) + 1);
        else
          return mailbox.Name;
      }
    }

    public ImapMailboxHierarchy[] Flatten()
    {
      var list = new List<ImapMailboxHierarchy>();

      list.Add(this);

      foreach (var hierarchy in subHierarchies) {
        list.AddRange(hierarchy.Flatten());
      }

      return list.ToArray();
    }

    internal ImapMailboxHierarchy(ImapMailbox mailbox, ImapMailboxHierarchy[] subHierarchies)
    {
      this.mailbox = mailbox;
      this.subHierarchies = subHierarchies;
    }

    internal static ImapMailboxHierarchy[] Create(ImapServerInfo serverInfo, IEnumerable<ImapMailbox> mailboxes)
    {
      // 階層の名前をツリー構造に変換する
      var nameTree = new HierarchyTree();

      foreach (ImapMailbox mailbox in mailboxes) {
        var hierarchyName = mailbox.Name.Split(new string[] {mailbox.HierarchyDelimiter}, StringSplitOptions.None);

        // 一番上の階層からたどってツリー構造を作る
        var tree = nameTree;

        foreach (var hierarchy in hierarchyName) {
          if (!tree.SubTrees.ContainsKey(hierarchy)) {
            tree.SubTrees.Add(hierarchy, new HierarchyTree());
          }

          tree = tree.SubTrees[hierarchy];
        }

        tree.Mailbox = mailbox;
      }

      // 変換したツリーからインスタンスを作成する
      return ParseHierarchyTree(nameTree.SubTrees.Values);
    }

    private static ImapMailboxHierarchy[] ParseHierarchyTree(IEnumerable<HierarchyTree> trees)
    {
      var hierarchies = new List<ImapMailboxHierarchy>();

      foreach (var tree in trees) {
        hierarchies.Add(new ImapMailboxHierarchy(tree.Mailbox, ParseHierarchyTree(tree.SubTrees.Values)));
      }

      return hierarchies.ToArray();
    }

    private readonly ImapMailboxHierarchy[] subHierarchies;
    private readonly ImapMailbox mailbox;
  }
}