using System;

namespace Smdn.Protocols.Imap4.Protocol {
  // ResponseTypes:
  //   ImapProtocolResponse(abstract base)
  //     => handles '7. Server Responses'
  //     ImapStatusResponse
  //       => handles '7.1. Server Responses - Status Responses'
  //     ImapDataResponse
  //       => handles '7.2. Server Responses - Server and Mailbox Status'
  //                  '7.3. Server Responses - Mailbox Size'
  //                  '7.4. Server Responses - Message Status'
  //     ImapCommandContinuationRequest
  //       => handles '7.5. Server Responses - Command Continuation Request'

  public class ImapStatusResponse : ImapProtocolResponse {
    public string Tag {
      get { return tag; }
    }

    public ImapProtocolStatus Status {
      get { return status; }
    }

    public ImapProtocolResponseCode ResponseCode {
      get { return responseCode; }
    }

    public string Text {
      get { return text; }
    }

    public ImapStatusResponse(string tag, ImapProtocolStatus status, ImapProtocolResponseCode responseCode, string text, string response)
      : base(response)
    {
      if (tag == null)
        throw new ArgumentNullException("tag");

      this.tag = tag;
      this.status = status;
      this.responseCode = responseCode;
      this.text = text;
    }

    public override string ToString()
    {
      return string.Format("{{Tag={0}, Status={1}, ResponceCode={2}, Text={3}}}", tag, status, responseCode, text);
    }

    private string tag = string.Empty;
    private ImapProtocolStatus status = ImapProtocolStatus.None;
    private ImapProtocolResponseCode responseCode = null;
    private string text = string.Empty;
  }
}