using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal abstract class ListTransactionBase : TransactionBase {

    // ProcessList
    //   ->WaitForList
    // WaitForList
    //   x Conclude

    protected ListTransactionBase(string command, ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
      commandString = command.ToLower();
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (Request.Parameters.ContainsKey("Mailbox")) {
        listMailbox = Request.Parameters["Mailbox"];

        if (string.Empty.Equals(listMailbox))
          listMailbox = "*";
        else
          listMailbox = string.Format("\"{0}\"", listMailbox);
      }
      else {
        listMailbox = "*";
      }

      return ProcessList;
    }

    private void ProcessList()
    {
      if ((Connection.ServerInfo.Capability.KnownCapabilities & ImapKnownCapabilities.Namespace) != 0) {
        // TODO: personal以外の名前空間にあるメールボックスの考慮
        lastTag = Send(commandString,
                       string.Format("\"{0}\"", Connection.ServerInfo.Namespace.Personal.Namespace),
                       listMailbox);
      }
      else {
        lastTag = Send(commandString, "\"\"", listMailbox);
      }

      Transit(WaitForList);
    }

    private void WaitForList()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        ImapMailboxFlags mailboxFlags;
        string[] extendedMailboxFlags;
        string hierarchyDelimiter;
        string name;

        if (ImapDataResponseConverter.FromListLsubResponse(response as ImapDataResponse,
                                                                   out mailboxFlags,
                                                                   out extendedMailboxFlags,
                                                                   out hierarchyDelimiter,
                                                                   out name)) {
          // get existing or created instance
          var mailbox = ImapMailboxManager.GetMailbox(Connection.ServerInfo, name);

          mailbox.MailboxFlags = mailboxFlags;
          mailbox.ExtendedMailboxFlags = extendedMailboxFlags;
          mailbox.HierarchyDelimiter = hierarchyDelimiter;

          Trace.Dump(mailbox);

          mailboxes.Add(mailbox);
        }
        else {
          Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "mailbox comprehension error occurred"));
        }
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != lastTag)
          return;

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapListResponse(ImapTransactionStatus.Ok, mailboxes.ToArray(), "mailbox listed"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag = string.Empty;
    private string commandString = string.Empty;
    private string listMailbox = string.Empty;
    private List<ImapMailbox> mailboxes = new List<ImapMailbox>();
  }
}
