using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class StatusTransaction : TransactionBase {

    // ProcessParameterNotSetted
    //   x Conclude
    // ProcessMailboxNotFound
    //   x Conclude
    // ProcessStatus
    //   ->WaitForStatus
    // WaitForStatus
    //   x Conclude

    public StatusTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Parameters.ContainsKey("Mailbox"))
        return ProcessParameterNotSetted;

      requestingMailbox = ImapMailboxManager.GetExistMailbox(Connection.ServerInfo, Request.Parameters["Mailbox"]);

      if (requestingMailbox == null)
        // TODO: LIST/LSUBで既知のメールボックスに対してしかSTATUSできない
        return ProcessMailboxNotFound;

      return ProcessStatus;
    }

    private void ProcessParameterNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "parameter 'Mailbox' must be setted"));
    }

    private void ProcessMailboxNotFound()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.NotFound, string.Format("mailbox '{0}' was not found", Request.Parameters["Mailbox"])));
    }

    private void ProcessStatus()
    {
      // TODO: \Noselectはstatusできない?  
      lastTag = Send("status",
                     string.Format("\"{0}\"", ImapConverter.ToModifiedUTF7String(requestingMailbox.Name)),
                     "(messages recent unseen uidvalidity)");

      Transit(WaitForStatus);
    }

    private void WaitForStatus()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        string name;
        int? messages;
        int? recent;
        int? unseen;
        int? uidNext;
        int? uidValidity;

        if (ImapDataResponseConverter.FromStatusResponse(response as ImapDataResponse,
                                                                 out name,
                                                                 out messages,
                                                                 out recent,
                                                                 out unseen,
                                                                 out uidNext,
                                                                 out uidValidity)) {
          if (requestingMailbox.Name.Equals(name)) {
            if (messages.HasValue)      requestingMailbox.ExistsMessage = (int)messages;
            if (recent.HasValue)        requestingMailbox.RecentMessage = (int)recent;
            if (unseen.HasValue)        requestingMailbox.UnseenMessage = (int)unseen;
            if (uidNext.HasValue)       requestingMailbox.UidNext = (int)uidNext;
            if (uidValidity.HasValue)   requestingMailbox.UidValidity = (int)uidValidity;
          }
          else {
            Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "status comprehension error occurred"));
          }
        }
        else {
          Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "status comprehension error occurred"));
        }
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != lastTag)
          return;

        Trace.Dump(requestingMailbox);

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapMailboxResponse(ImapTransactionStatus.Ok, requestingMailbox, "status succeeded"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag = string.Empty;
    private ImapMailbox requestingMailbox;
  }
}
